#!/usr/bin/python
#
# Package list mangling
#
# $Id: packageList.py,v 1.37 2003/02/04 21:53:52 alikins Exp $

import os
import fnmatch
import string
import sys
import rpm
# temp home for all the package list related functions
# move this to a class
import up2dateUtils
import up2dateLog
import up2dateErrors
import config
import rpmUtils
import headers
import rhnPackageInfo
import transaction

from rhpl.translate import _

#grab an instance of the config class shared
cfg = config.initUp2dateConfig()

# get a reference to the log file object
log = up2dateLog.initLog()

headerList = headers.HeaderList()


# idea... a class to wrap everything in here...
#   we add all the info we need, and then call PackageList.getTransaction or
#    PackageList.runTransaction or something
#
#   needs to be able to handle adding packages for install/update/remove
#   and some set of standard options for those (--force?, --nodeps, --justdb,etc)
#
#   At the very least, this needs to be able to do what getUpdatedPackageList does
#
#
class PackageList:
    def __init__(self):
        # rpm transaction set flags
        self.nodeps = None
        self.force = None
        self.justdb = None
        # A hash keyed on the package name, and with a list of all the
        # packages with that name as a value
        self.__instPackagesHash = {}
        # A list of available packages
        self.__availPackagesList = []
        # A hash keyed on the package name, None as value
        self.__forcePackagesHash = {}
        # A hash keyed on the package (n, v, r, e, a), a list of
        # (n, v, r, e, a, obsol name, obsol ver, obsol sense) as values
        self.__obsolPackagesHash = {}
        # list of packages we should update but are marked to
        # be skipped for some reason. for ui stuff
        # (names with possible wildcards)
        self.__skipPatternList = []
        # Skipped packages
        self.__skippedPackagesList = []
        self.__obsoletedPackagesList = []
        self.__installedObsoletingPackageList = []

        self.msgCallback = None
        self.progressCallback = None
        self.refreshCallback = None
        # bit of a kluge to make a better ui
        self.ignoreMsgCallback = 1

    def run(self):
        # based on getpdatedPackageList
        installedPackageList = rpmUtils.getInstalledPackageList()
        self.addInstalledPackages(installedPackageList)
        availList = rhnPackageInfo.availablePackageList(self.msgCallback,
                                                        self.progressCallback)

        if availList:
            self.addAvailablePackages(availList)

        obsList = rhnPackageInfo.obsoletesList(
            msgCallback = self.msgCallback,
            progressCallback = self.progressCallback )
        self.addObsoletePackages(obsList)

        if not cfg["forceInstall"]:
            self.addSkipPatterns(cfg["pkgSkipList"])

        
        # skipped packages and the packages to install
        # are self.getSkippedPackages and self.getPackagesToInstall

        
        
    def addInstalledPackages(self, installedPackages):
        for p in installedPackages:
            pName = p[0]
            if not self.__instPackagesHash.has_key(pName):
                self.__instPackagesHash[pName] = []
            self.__instPackagesHash[pName].append(p)

    def addAvailablePackages(self, availablePackages):
        for p in availablePackages:
            self.__availPackagesList.append(p)
        #print self.__availPackagesList

    def addForcePackages(self, forcePackages):
        for p in forcePackages:
            self.__forcePackagesHash[p] = None

    def addObsoletePackages(self, obsoletePackages):
        for p in obsoletePackages:
            key = tuple(p[:5])
            if not self.__obsolPackagesHash.has_key(key):
                self.__obsolPackagesHash[key] = []
            self.__obsolPackagesHash[key].append(p)
            
        self.__obsoletedPackageHash = {}
        for key in self.__obsolPackagesHash.keys():
            for obsInfo in self.__obsolPackagesHash[key]:            
                if not self.__obsoletedPackageHash.has_key(obsInfo[5]):
                    self.__obsoletedPackageHash[obsInfo[5]] = []
                    self.__obsoletedPackageHash[obsInfo[5]].append(obsInfo)
            # foo[obsoletingpackage] = (obs info, with [5] being the thing obsoleted)
            # ugh

    def addSkipPatterns(self, skipPatterns):
        for p in skipPatterns:
            self.__skipPatternList.append(p)

    def getInstalledPackages(self):
        return self.__instPackagesHash

    def getForcedPackages(self):
        return self.__forcePackagesHash
    
    def getObsoletedPackages(self):
        return self.__obsoletedPackagesList

    def getInstalledObsoletingPackages(self):
        return self.__installedObsoletingPackageList
    


    def getSkippedPackages(self):
        """
        Returns a list of packages that have been skipped, together with the
        reason why they've been skipped
        """
        # XXX
        return self.__skippedPackagesList

    # Is this package outdated by a an already installed package?
    def __outdatedByInstalledPackage(self, p):
        for inst in self.__instPackagesHash[p[0]]:
            ret = up2dateUtils.comparePackages(inst, p)
            if ret < 0:
                log.log_debug("p: %s is newer than inst: %s" % (
                    p, inst))
            else:
                # at least one of the installed versions is
                # new enough
                return 1

        # No installed package is newer than p
        return 0

    def __findOutdatedPackages(self):
        for p in self.__availPackagesList:
            pName = p[0]

            # is this package already installed?
            if self.__instPackagesHash.has_key(pName):
                # if so, we need to do version compare stuff

                # Installed
                # If the package is not outdated by an already installed
                # package, mark it as available for updates
                if not self.__outdatedByInstalledPackage(p):
                    self.__addPackageToUpdate(p)

    # is the package one of the packages we are installing/force upgrading?
    def __findForcedPackages(self):
        fpHash = self.__forcePackagesHash
        for p in self.__availPackagesList:
            pName = p[0]

            if self.packagesToUpdate.has_key(pName):
                continue

            if fpHash.has_key(pName):
                # if a version of the package is installed
                if self.__instPackagesHash.has_key(pName):
                    # and the installed version isnt the same or newer than
                    # what we are forcing
                    if not self.__outdatedByInstalledPackage(p):
                        self.__addPackageToUpdate(p)
                else:
                    self.__addPackageToUpdate(p)
#                self.packagesToUpdate[pName] = []
#                self.packagesToUpdate[pName].append(p)

    def __addPackageToUpdate(self,p):
        pName = p[0]
        if not self.packagesToUpdate.has_key(pName):
            self.packagesToUpdate[pName] = []
        self.packagesToUpdate[pName].append(p)


    def __findObsoletingPackages(self):
        for p in self.__availPackagesList:
            pName = p[0]

            if self.packagesToUpdate.has_key(pName):
                # we're already updating this package
                continue

            key = tuple(p[:5])
            if not self.__obsolPackagesHash.has_key(key):
                # Nothing to do here
                continue

            # this package obsoletes something 
            for obs in self.__obsolPackagesHash[key]:
                obsName, obsVersion, obsSense = obs[5:]
                if obsSense == "0":
                    # this package obsoletes all versions of
                    # obsName
                    if not self.__instPackagesHash.has_key(obsName):
                        continue
                    self.__addPackageToUpdate(p)
                    # any given one package can obsolete multiple packages
                    for obsP in self.__instPackagesHash[obsName]:
                        self.__obsoletedPackagesList.append((obsP,self.__obsoletedPackageHash[obsName]))
                    continue

                # Have to compare versions
                vv, rr, ee = parseObsoleteVersion(obsVersion)
                candidate = (obsName, vv, rr, ee)
                # loop over all the installed versions
                if not self.__instPackagesHash.has_key(obsName):
                    # Nothing to do
                    continue
                
                # obsName is installed
                for inst in self.__instPackagesHash[obsName]:
                    ret = up2dateUtils.comparePackages(inst, candidate)
                    op = cmp2rpmSense(ret)
                    if op & int(obsSense):
                        # this is a package that obsoletes the version
                        # we have...

                        self.__addPackageToUpdate(p)
                        for obsP in self.__instPackagesHash[obsName]:
                            self.__obsoletedPackagesList.append((obsP,self.__obsoletedPackageHash[obsName]))

    def __findBestArch(self, plist):
        bestArchP = None
        for p in plist:
            archscore = rpm.archscore(p[4])
            log.log_debug("archscore", archscore)
            if archscore <= 0:
                # Unsupported architecture
                continue
            if bestArchP and rpm.archscore(bestArchP[4]) <= archscore:
                # The currently best arch is better than (or the same as) 
                # this one
                continue

            # This is a better architecture
            bestArchP = p
                
        return bestArchP

            
    def __findBestArchPackages(self):
        hash = {}
        for pkey, plist in self.packagesToUpdate.items():
            newp = self.__findBestArch(plist)
            # newp can be None at this point, if no packages apply to this
            # system
            if newp:
                self.packagesToUpdate[pkey] = newp


    def __sortPackageList(self):
        result = self.packagesToUpdate.values()
        result.sort(lambda a, b: cmp(a[0], b[0]))
        return result
            
    def getPackagesToInstall(self):
        # We put stuff in a hash to easily find packages
        hash = {}

        # hash of packages we need to update
        self.packagesToUpdate = {}

        self.__findOutdatedPackages()
        self.__findForcedPackages()
        self.__findObsoletingPackages()
        self.__findBestArchPackages()
#        self.__handleForcePackages()
        result = self.__sortPackageList()
        result = self.__skipPackages(result)
        return result



    # list of failed deps if any
    # for ui stuff
    def failedDeps(self):
        pass

    # either return a rpm.TransactionSet object
    # or some similar representation
    def getTransaction(self):
        pass

    def __skipPackages(self, packageList):
        self.__skippedPackagesList = []

        for p_i in range(len(packageList)):
            for pattern in self.__skipPatternList:
                if fnmatch.fnmatch(packageList[p_i][0], pattern):
                    self.__skippedPackagesList.append((packageList[p_i], _("Pkg name/pattern")))

                    break

        for p in self.__skippedPackagesList:
            packageList.remove(p[0])

        packageList = self.__skipFiles(packageList)
        return packageList

    def __skipFiles(self, packageList):
        removedList = []
        if cfg['forceInstall']:
            return packageList

        # need the db for the checkHeaderFiles
	ts = transaction.initReadOnlyTransaction()
#        ts = rpm.TransactionSet()
#	ts.setVerifySigFlags(8)
 
        p_i = 0
	# FIXME: something seems wrong here, when we are
	# are checking for FileConfigExcludes, we try to load the
	# header of the installed version, which doesnt make sence
	# for the new package case
        h_list = buildHeaderList(packageList, msgCallback = self.msgCallback,
                                 progressCallback = self.progressCallback,
                                 refreshCallback = self.refreshCallback,
                                 ignoreMsgCallback = self.ignoreMsgCallback)
        for i in h_list:
            tmp_list = None
            tmp_list = rpmUtils.checkHeaderForFileConfigExcludes(i,packageList[p_i],ts)
            p_i = p_i + 1
            if tmp_list:
   	         removedList = removedList + tmp_list


        d = {}
        for i in removedList:
            d[tuple(i[0])] = i
        removedList = d.values()

        for p in removedList:
            packageList.remove(p[0])

        self.__skippedPackagesList = self.__skippedPackagesList + removedList
        return packageList


def parseObsoleteVersion(ver):
    s = ver
    epoch = ""
    if string.find(s, ":") >= 0:
        arr = string.split(s, ":", 1)
        epoch = arr[0]
        s = arr[1]
    release = "0"
    if string.find(s, '-') >= 0:
        arr = string.split(s, "-", 1)
        release = arr[1]
        s = arr[0]
    return s, release, epoch
        

def cmp2rpmSense(value):
    if value < 0:
        return rpm.RPMSENSE_LESS
    if value > 0:
        return rpm.RPMSENSE_GREATER
    return rpm.RPMSENSE_EQUAL

def getArchesOfAvailablePackages(availablePkgList,pkgName):
    archList = []
    for pkg in availablePkgList:
        if pkgName == pkg[0]:
            archList.append(pkg[4])

    return archList

def buildHeaderList(pkgList,msgCallback = None, progressCallback = None,
                            refreshCallback = None, ignoreMsgCallback=None):
    h_list = []
    count = 1
    total = len(pkgList)
    if msgCallback:
        msgCallback(_("Fetching rpm headers"))
    for pkg in pkgList:
        hdr = headerList[pkg]
        if hdr:
            h_list.append(hdr)
        # I dont want to do this in the batch mode, but it's nice
        # for the gui. Ugly, but better ui...
        if msgCallback and not ignoreMsgCallback:
            msgCallback(_("Fetching rpm header: %s-%s-%s") % (hdr['name'],hdr['version'],hdr['release']))
        if progressCallback:
            progressCallback(count, total)
        count = count + 1 
    return h_list
            

# side effects on pkgList
# REFACTOR: this probabaly needs to stay module scope, as
# depSolver.__skip calls it
def removeSkipPackagesFromList(pkgList, msgCallback = None,
                               progressCallback = None):
    removedList = []
    pkgCount = len(pkgList)

    if cfg["forceInstall"]:
        return removedList

    if msgCallback != None:
        msgCallback(_("Removing packages marked to skip from list"))

    
    pkgSkipList = cfg["pkgSkipList"]
    if cfg["forceInstall"]:
        pkgSkipList = []
        
    for index in range(pkgCount):
        for pattern in pkgSkipList:
            if fnmatch.fnmatch(pkgList[index][0], pattern):
                removedList.append((pkgList[index], _("Pkg name/pattern")))
                break

        if progressCallback != None:
            progressCallback(index+1, pkgCount)

    for pkg in removedList:
        pkgList.remove(pkg[0])

    # just do a side effect on pkgList here, and return removedList
    # since we actually need the results of it
    return removedList


if __name__ == '__main__':
    installedPackageList = []
    pList = PackageList()
    for i in range(20):
        installedPackageList.append("a%04d" % i, '1.0', '1', '', 'noarch')
    pList.addInstalledPackages(installedPackageList)
    availablePackageList = []
    for i in range(30):
        for arch in ['noarch', 'i386', 'i586', 'i686', 'athlon']:
            availablePackageList.append("a%04d" % i, '2.0', '1', '', arch)

    print len(installedPackageList), len(availablePackageList)
    pList.addAvailablePackages(availablePackageList)

    forcePackageList = []
    for i in range(10):
        forcePackageList.append("a%04d" % (20 + 5 * i))
    
#    pList.addForcePackages(forcePackageList)

    obsList = [
        ('a0021', '2.0', '1', '', 'i686', 'a0019', '', 0),
        ('a0022', '2.0', '1', '', 'i686', 'a0018', '2.0-1', 2),
    ]
    pList.addObsoletePackages(obsList)

    pList.addSkipPatterns(["a0017", "a001[3-5]"])
    
    import time
    start = time.time()
    res = pList.getPackagesToInstall()
    print "Total time: %f" % (time.time() - start)
    print len(res)
    print res
